import { Observable } from 'rxjs';
import { Arc, IApolloQueryOptions } from './arc';
import { Address, ICommonQueryOptions, IStateful } from './types';
export interface IEventState {
    id: string;
    dao: string;
    proposal: string;
    user: string;
    type: string;
    data: {
        [key: string]: any;
    };
    timestamp: string;
}
export interface IEventQueryOptions extends ICommonQueryOptions {
    where?: {
        id?: string;
        dao?: Address;
        proposal?: string;
        user?: Address;
        [key: string]: any;
    };
}
export declare class Event implements IStateful<IEventState> {
    context: Arc;
    idOrOpts: string | IEventState;
    static fragments: {
        EventFields: import("graphql").DocumentNode;
    };
    /**
     * Event.search(context, options) searches for reward entities
     * @param  context an Arc instance that provides connection information
     * @param  options the query options, cf. IEventQueryOptions
     * @return         an observable of Event objects
     */
    static search(context: Arc, options?: IEventQueryOptions, apolloQueryOptions?: IApolloQueryOptions): Observable<Event[]>;
    id: string;
    coreState: IEventState | undefined;
    constructor(context: Arc, idOrOpts: string | IEventState);
    state(apolloQueryOptions?: IApolloQueryOptions): Observable<IEventState>;
    setState(opts: IEventState): void;
    fetchState(apolloQueryOptions?: IApolloQueryOptions): Promise<IEventState>;
}
