import BN = require('bn.js');
import { Observable } from 'rxjs';
import { Arc, IApolloQueryOptions } from './arc';
import { DAO } from './dao';
import { IProposalQueryOptions, Proposal } from './proposal';
import { Reward } from './reward';
import { IStakeQueryOptions, Stake } from './stake';
import { Address, ICommonQueryOptions, IStateful } from './types';
import { IVoteQueryOptions, Vote } from './vote';
export interface IMemberState {
    address: Address;
    dao?: Address;
    contract?: Address;
    id?: string;
    reputation: BN;
}
export interface IMemberQueryOptions extends ICommonQueryOptions {
    where?: {
        id?: string;
        address?: Address;
        dao?: Address;
    };
}
/**
 * Represents an account that holds reputaion in a specific DAO
 */
export declare class Member implements IStateful<IMemberState> {
    context: Arc;
    static fragments: {
        ReputationHolderFields: import("graphql").DocumentNode;
    };
    /**
     * Member.search(context, options) searches for member entities
     * @param  context an Arc instance that provides connection information
     * @param  options the query options, cf. IMemberQueryOptions
     * @return         an observable of IRewardState objects
     */
    static search(context: Arc, options?: IMemberQueryOptions, apolloQueryOptions?: IApolloQueryOptions): Observable<Member[]>;
    static calculateId(opts: {
        contract: Address;
        address: Address;
    }): string;
    id: string | undefined;
    coreState: IMemberState | undefined;
    /**
     * @param address addresssof the member
     * @param daoAdress addresssof the DAO this member is a member of
     * @param context an instance of Arc
     */
    constructor(context: Arc, idOrOpts: string | IMemberState);
    fetchState(apolloQueryOptions?: IApolloQueryOptions): Promise<IMemberState>;
    setState(opts: IMemberState): IMemberState;
    state(apolloQueryOptions?: IApolloQueryOptions): Observable<IMemberState>;
    dao(): Promise<DAO>;
    rewards(): Observable<Reward[]>;
    proposals(options?: IProposalQueryOptions, apolloQueryOptions?: IApolloQueryOptions): Observable<Proposal[]>;
    stakes(options?: IStakeQueryOptions, apolloQueryOptions?: IApolloQueryOptions): Observable<Stake[]>;
    votes(options?: IVoteQueryOptions, apolloQueryOptions?: IApolloQueryOptions): Observable<Vote[]>;
}
