import { Observable } from 'rxjs';
import { Arc, IApolloQueryOptions } from './arc';
import { DAO } from './dao';
import { ISchemeState } from './scheme';
import { Address, ICommonQueryOptions, IStateful } from './types';
export interface IQueueState {
    dao: DAO;
    id: string;
    name: string;
    scheme: ISchemeState;
    threshold: number;
    votingMachine: Address;
}
export interface IQueueQueryOptions extends ICommonQueryOptions {
    where?: {
        dao?: Address;
        votingMachine?: Address;
        scheme?: Address;
        [key: string]: any;
    };
}
export declare class Queue implements IStateful<IQueueState> {
    context: Arc;
    id: string;
    dao: DAO;
    /**
     * Queue.search(context, options) searches for queue entities
     * @param  context an Arc instance that provides connection information
     * @param  options the query options, cf. IQueueQueryOptions
     * @return         an observable of Queue objects
     */
    static search(context: Arc, options?: IQueueQueryOptions, apolloQueryOptions?: IApolloQueryOptions): Observable<Queue[]>;
    constructor(context: Arc, id: string, dao: DAO);
    fetchState(apolloQueryOptions?: IApolloQueryOptions): Promise<IQueueState>;
    state(apolloQueryOptions?: IApolloQueryOptions): Observable<IQueueState>;
}
